webpackJsonp([2],{"44lp":function(a,n){document.write("\n  \x3c!-- Global site tag (gtag.js) - Google Analytics --\x3e\n  <script async src=\"https://www.googletagmanager.com/gtag/js?id=UA-132758957-2\"><\/script>\n  <script>\n    window.dataLayer = window.dataLayer || [];\n    function gtag(){dataLayer.push(arguments);}\n    gtag('js', new Date());\n\n    gtag('config', 'UA-132758957-2');\n  <\/script>\n")}},["44lp"]);
//# sourceMappingURL=analytics.7727105aa83964c1b62e.js.map