#nprogress
  pointer-events none
  .bar
    background $accentColor
    position fixed
    z-index 1031
    top 0
    left 0
    width 100%
    height 2px
  .peg
    display block
    position absolute
    right 0px
    width 100px
    height 100%
    box-shadow 0 0 10px $accentColor, 0 0 5px $accentColor
    opacity 1.0
    transform rotate(3deg) translate(0px, -4px)
  .spinner
    display block
    position fixed
    z-index 1031
    top 15px
    right 15px
  .spinner-icon
    width 18px
    height 18px
    box-sizing border-box
    border solid 2px transparent
    border-top-color $accentColor
    border-left-color $accentColor
    border-radius 50%
    animation nprogress-spinner 400ms linear infinite

.nprogress-custom-parent
  overflow hidden
  position relative

.nprogress-custom-parent #nprogress .spinner,
.nprogress-custom-parent #nprogress .bar
  position absolute

@keyframes nprogress-spinner
  0%
    transform rotate(0deg)
  100%
    transform rotate(360deg)
