{% extends "security/base.html" %}
{% from "security/_macros.html" import render_field_with_errors, render_field, title %}

{% block content %}
{% include "security/_messages.html" %}
{{ title(_('Resend confirmation instructions')) }}
<form action="{{ url_for_security('send_confirmation') }}" method="POST" name="send_confirmation_form">
  {{ send_confirmation_form.hidden_tag() }}
  {{ render_field_with_errors(send_confirmation_form.email) }}
  {{ render_field(send_confirmation_form.submit, class_="button is-link") }}
</form>
{% endblock %}
